"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
const os = __importStar(require("node:os"));
const oldFs = __importStar(require("node:fs"));
const colors_1 = __importDefault(require("colors"));
const restify = __importStar(require("restify"));
const errors = __importStar(require("restify-errors"));
const yargs = __importStar(require("yargs"));
const filesize_1 = require("filesize");
const qr = __importStar(require("./qr"));
const utils_1 = require("./utils");
const fs = oldFs.promises;
const argv = yargs
    .command("$0 <fileName>", "Receive a file.", args => args.positional("fileName", { type: "string" }))
    .option("port", {
    alias: "p",
    type: "number",
    default: 8080,
})
    .option("maxFileSize", {
    type: "number",
    default: 10 * 1024 * 1024 * 1024, // 10GiB
})
    .option("hashingFunction", {
    type: "string",
    describe: "Get a list via `openssl list -digest-algorithms`",
    default: "sha256",
})
    .option("tempDir", {
    type: "string",
    describe: "Directory to store the file temporarily",
    default: os.tmpdir(),
})
    .option("noEmptyFiles", {
    type: "boolean",
    describe: "If the user uploads an empty file, dismiss it.",
    default: false,
})
    .option("overwrite", {
    type: "boolean",
    describe: "Overwrite <fileName> if it already exists.",
    default: true,
})
    .option("noToken", {
    type: "boolean",
    describe: "Just use a link without any session-specific token.",
    default: undefined,
    conflicts: ["token"],
})
    .option("token", {
    type: "string",
    describe: "The session-specific token to use. Will be generated randomly if omitted.",
    default: undefined,
    conflicts: ["noToken"],
})
    .option("note", {
    type: "string",
    describe: "leave a note for the sender. Will be displayed in the browser.",
    default: "",
})
    .option("fileName", {
    type: "string",
    describe: "Target file name.",
})
    .require("fileName")
    .string("fileName")
    .help()
    .alias("h", "help")
    .wrap(yargs.terminalWidth())
    .argv;
const formatFileSize = (0, filesize_1.partial)({ standard: "iec" });
const token = !!argv.noToken
    ? ""
    : ((_a = argv.token) !== null && _a !== void 0 ? _a : (0, utils_1.randomString)());
const server = restify.createServer({
    name: "send-me-a-file",
});
const hashingFunction = argv.hashingFunction.toLowerCase();
const protocol = "http";
server.use(restify.plugins.multipartBodyParser({
    hash: hashingFunction,
    maxFileSize: argv.maxFileSize,
    uploadDir: argv.tempDir,
    multiples: false,
}));
server.get("/:token", async (req, res, next) => {
    // TODO: Assertion function
    if (req.params.token !== token) {
        return next(new errors.BadRequestError("Invalid token provided."));
    }
    res.writeHead(200);
    if ((0, utils_1.isRequestingFromBrowser)(req)) {
        const indexTemplate = await (0, utils_1.readPublicFile)("index.html");
        const htmlNote = !!argv.note
            ? "<h2>Note from receiver</h2>\n" + argv.note
            : "";
        const href = (0, utils_1.getServerUrlFromRequest)(protocol, req, token, "<script>document.write(document.location.href);</script>");
        const index = indexTemplate
            .replace(/%note%/gi, htmlNote)
            .replace(/%host%/gi, href);
        res.contentType = "text/html";
        res.end(index);
    }
    else {
        const note = !!argv.note
            ? "\n" + colors_1.default.dim("Note from the receiver:\n") + colors_1.default.bold(argv.note)
            : "";
        const href = (0, utils_1.getServerUrlFromRequest)(protocol, req, token, "<this address>:<port>");
        const content = [
            colors_1.default.yellow("Someone requested a file from you!"),
            "",
            colors_1.default.dim("You can simply use curl to upload it:"),
            `  ${colors_1.default.bold(`curl "${href}" -F file=@/path/to/file.zip`)}`,
            "",
            "...or open this URL in your browser.",
            note,
        ].join("\n");
        res.contentType = "text/plain";
        res.end((0, utils_1.indentText)(content, "  ") + "\n");
    }
    return next();
});
server.post("/:token", async (req, res, next) => {
    var _a;
    // TODO: Assertion function
    if (req.params.token !== token) {
        return next(new errors.BadRequestError("Invalid token provided."));
    }
    // TODO: Enhance restify.RequestFileInterface
    const uploadedFile = (_a = req.files) === null || _a === void 0 ? void 0 : _a.file;
    if (!uploadedFile) { // TODO: Assertion function
        return next(new errors.BadRequestError("No file uploaded"));
    }
    if (argv.noEmptyFiles && uploadedFile.size <= 0) {
        console.error("User uploaded an empty file, skipping.");
        return next(new errors.BadRequestError("Uploaded empty file."));
    }
    await checkForFileOverwrite(false);
    await fs.rename(uploadedFile.path, argv.fileName);
    console.log("Someone uploaded a file!");
    const info = [
        {
            name: "Size",
            content: `${uploadedFile.size} bytes (${formatFileSize(uploadedFile.size)})`,
        }, {
            name: "Local path",
            content: argv.fileName,
        }, {
            name: "Client-Supplied file name",
            content: uploadedFile.name,
        }, {
            name: "Type",
            content: uploadedFile.type,
        }, {
            name: hashingFunction + " hash",
            content: uploadedFile.hash,
        }, {
            name: "Last modified",
            content: uploadedFile.lastModifiedDate,
        },
    ];
    const longestKeyLength = Math.max(...info.map(i => i.name.length));
    console.log();
    info.forEach(i => {
        console.log(formatInfo(i, longestKeyLength + 4));
    });
    console.log();
    console.log("Bye!");
    res.writeHead(200);
    if ((0, utils_1.isRequestingFromBrowser)(req)) {
        res.contentType = "text/html";
        const thanks = await (0, utils_1.readPublicFile)("thanks.html");
        res.end(thanks);
    }
    else {
        res.contentType = "text/plain";
        const info = [
            {
                name: `${hashingFunction} hash of the file received`,
                content: uploadedFile.hash,
            }, {
                name: "Size",
                content: `${uploadedFile.size} bytes (${formatFileSize(uploadedFile.size)})`,
            },
        ];
        const longestNameLength = Math.max(...info.map(i => i.name.length));
        const textReply = [
            colors_1.default.green("Thanks for the file!"),
            "",
            ...info.map(i => formatInfo(i, longestNameLength + 4)),
            "",
            "Have a nice day!",
            "",
        ].join("\n"); // We use this instead of `` because the line encoding of this file could change.
        res.end(textReply);
    }
    return process.exit();
});
// TODO: Implement these options:
// TODO: If dest-file-name is stdout, print file to stdout
// TODO: accept only same file extension
// TODO: TLS with public key pinning via curl
// TODO: Option to trust user-supplied file
async function main() {
    await checkForFileOverwrite(true);
    console.log("Starting local http server...");
    if (argv.tempDir !== os.tmpdir()) {
        console.log(`Using "${argv.tempDir}" as a temporary directory for file uploads.`);
    }
    console.log();
    server.listen(argv.port, async () => {
        const interfaces = os.networkInterfaces();
        const validInterfaces = Object.values(interfaces)
            .flat()
            .filter(iface => (iface === null || iface === void 0 ? void 0 : iface.family) === "IPv4");
        for (const iface of validInterfaces) {
            await printEndpoint(protocol, iface, argv.port, token);
        }
        console.log(colors_1.default.yellow(`Waiting for someone to upload ${colors_1.default.blue(argv.fileName)}`));
        console.log();
    });
}
main();
function formatInfo(info, startPadding) {
    const valueToPrint = typeof info.content === "undefined"
        ? "<undefined>"
        : typeof info.content === "string"
            ? info.content
            : info.content.toString();
    return colors_1.default.dim(info.name.padStart(startPadding) + ": ") + colors_1.default.bold(valueToPrint);
}
async function printEndpoint(protocol, iface, port, token) {
    console.log(`  ${protocol}://${iface.address}:${colors_1.default.green(port.toString())}/${token}`);
    if ((0, utils_1.isLoopback)(iface))
        return;
    // Don't print QRcodes to localhost
    const terimalQrCode = await qr.terminal(`${protocol}://${iface.address}:${port}/${token}`);
    const indentedQrCode = (0, utils_1.indentText)(terimalQrCode, "    ");
    console.log();
    console.log("    " + colors_1.default.dim("Upload via cURL:"));
    console.log(colors_1.default.bold(`    curl "${protocol}://${iface.address}:${port}/${token}" -F file=@/path/to/file.zip`));
    console.log();
    console.log(indentedQrCode);
    console.log();
}
async function checkForFileOverwrite(print) {
    const fileExists = await fs.access(argv.fileName, oldFs.constants.F_OK | oldFs.constants.W_OK)
        .then(() => true)
        .catch(() => false);
    if (fileExists) {
        if (argv.overwrite) {
            print && console.warn(`File ${argv.fileName} already exists. It will be overridden.`);
        }
        else {
            print && console.warn(`File ${argv.fileName} already exists.`);
            return process.exit(-1);
        }
    }
}
